//[skeleton1
package io;
import java.io.*;
import java.net.*;
import java.util.*;


/**
 * A ResourceFinder is used to find a "resource" either in a .jar file or 
 * in the local file system. It uses a ClassLoader to find the "resource".
 *
 * @author  Prof. David Bernstein, James Madison University
 * @see     "The Design and Implementation of Multimedia Software  2011"
 * @version 1.0
 */
public class ResourceFinder
{
  private static final Map<Class<?>, ResourceFinder> pool = new HashMap<Class<?>, ResourceFinder>();
  
  private final Class<?>  c;


  /**
   * Explicit Value Constructor.
   *
   * @param   The Class 
   */
  private ResourceFinder(final Class<?> c)
  {
    this.c = c;
  }

  /**
   * Construct a ResourceFinder that will search "locally".
   *
   * @return   A ResourceFinder that will search "locally"
   */
  public static ResourceFinder createInstance()
  {
    return createInstance(ResourceFinder.class);
  }

  /**
   * Construct a ResourceFinder that will search "near" a particular Object.
   *
   * @param o  The Object that defines the location of the search
   * @return   A ResourceFinder that will search "near" o
   */
  public static ResourceFinder createInstance(final Object o)
  {
    return createInstance(o.getClass());
  }

  /**
   * Construct a ResourceFinder that will search "near" a particular Class.
   *
   * @param c  The Class that defines the location of the search
   * @return   A ResourceFinder that will search "near" o
   */
  public static ResourceFinder createInstance(final Class<?> c)
  {
    ResourceFinder result = pool.get(c);
    if (result == null)
    {
      result = new ResourceFinder(c);
      pool.put(c, result);
    }
    
    return result;
  }
//]skeleton1

//[findInputStream
    /**
     * Find a resource.
     *
     * @return  The InputStream of the resource (or null)
     */
    public InputStream findInputStream(String name)
    {
       InputStream    is;

       is    = c.getResourceAsStream(name);

       return is;       
    }
//]findInputStream

//[findURL
    /**
     * Find a resource.
     *
     * @return  The URL of the resource (or null)
     */
    public URL findURL(String name)
    {
       URL            url;

       url   = c.getResource(name);
       
       return url;       
    }
//]findURL

//[loadResourceNames
    /**
     * Load a list of resource names from a list (e.g., file).
     *
     * Note: This method does not return an array of InputStream
     * objects to conserver resources.
     *
     * @param  listName  The name of the list
     * @return           The resource names
     */
    public String[] loadResourceNames(String listName)
    {
       ArrayList<String>        buffer;       
       BufferedReader           in;       
       InputStream              is;
       String                   line;       
       String[]                 names;
       
       names = null;       
       is    = findInputStream(listName);

       if (is != null)
       {
          try
          {
             in     = new BufferedReader(new InputStreamReader(is));
             buffer = new ArrayList<String>();
             
             while ((line=in.readLine()) != null)
             {
                buffer.add(line);
             }
             names = new String[buffer.size()];             
             buffer.toArray(names);             
          }
          catch (IOException ioe)
          {
             // Can't read the list
          }
       }
       return names;       
    }
//]loadResourceNames
//[skeleton2
}
//]skeleton2
